/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright© 2020 MikroElektronika d.o.o.
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be 
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE 
 * OR OTHER DEALINGS IN THE SOFTWARE. 
 */

/*!
 * \file
 *
 */

#include "tiltnshake.h"

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void tiltnshake_cfg_setup ( tiltnshake_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->scl = HAL_PIN_NC;
    cfg->sda = HAL_PIN_NC;
    
    // Additional gpio pins

    cfg->en   = HAL_PIN_NC;
    cfg->int_pin = HAL_PIN_NC;

    cfg->i2c_speed = I2C_MASTER_SPEED_STANDARD; 
    cfg->i2c_address = TILTNSHAKE_MMA8491Q_I2C;
}

TILTNSHAKE_RETVAL tiltnshake_init ( tiltnshake_t *ctx, tiltnshake_cfg_t *cfg )
{
    i2c_master_config_t i2c_cfg;

    i2c_cfg.speed  = cfg->i2c_speed;
    i2c_cfg.scl    = cfg->scl;
    i2c_cfg.sda    = cfg->sda;

    ctx->slave_address = cfg->i2c_address;

    if ( i2c_master_open( &ctx->i2c, &i2c_cfg ) == I2C_MASTER_ERROR )
    {
        return TILTNSHAKE_INIT_ERROR;
    }

    i2c_master_set_slave_address( &ctx->i2c, ctx->slave_address );
    i2c_master_set_speed( &ctx->i2c, cfg->i2c_speed );

    // Output pins 

    digital_out_init( &ctx->en, cfg->en );

    // Input pins

    digital_in_init( &ctx->int_pin, cfg->int_pin );

    return TILTNSHAKE_OK;

}

void tiltnshake_generic_write ( tiltnshake_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    uint8_t tx_buf[ 256 ];
    uint8_t cnt;
    
    tx_buf[ 0 ] = reg;
    
    for ( cnt = 1; cnt <= len; cnt++ )
    {
        tx_buf[ cnt ] = data_buf[ cnt - 1 ]; 
    }
    
    i2c_master_write( &ctx->i2c, tx_buf, len + 1 );  
}

void tiltnshake_generic_read ( tiltnshake_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_write_then_read( &ctx->i2c, &reg, 1, data_buf, len );
}

void tiltnshake_enable ( tiltnshake_t *ctx, uint8_t en_dev )
{
    digital_out_write( &ctx->en, en_dev );
    Delay_10ms( );
}

void tiltnshake_read_status_and_axis ( tiltnshake_t *ctx, uint8_t *status, uint16_t *out_x, uint16_t *out_y, uint16_t *out_z )
{
   uint8_t temp[ 7 ];
   Delay_1ms( );
   tiltnshake_generic_read( ctx, TILTNSHAKE_STATUS_REG, temp, 7 );
   *status = temp[ 0 ];
   *out_x = ( ( 0x100 * temp[ 1 ] + temp[ 2 ] ) >> 2 ) & 0x3FFF;
   *out_y = ( ( 0x100 * temp[ 3 ] + temp[ 4 ] ) >> 2 ) & 0x3FFF;
   *out_z = ( ( 0x100 * temp[ 5 ] + temp[ 6 ] ) >> 2 ) & 0x3FFF;
   Delay_1ms( );
}

void tiltnshake_conversion ( uint16_t *out_x, uint16_t *out_y, uint16_t *out_z, float *out_x_flo, float *out_y_flo, float *out_z_flo )
{
   if ( *out_x & 0x2000 ) 
   {
       *out_x_flo = -( ( 0x4000 - *out_x ) / 1024.0 );
       asm nop
   }
   else 
   {
       *out_x_flo = *out_x / 1024.0;
       asm nop
   }
   if ( *out_y & 0x2000 ) 
   {
       *out_y_flo = -( ( 0x4000 - *out_y ) / 1024.0 );
       asm nop
   }
   else 
   {
       *out_y_flo = *out_y / 1024.0;
       asm nop
   }
   if ( *out_z & 0x2000 ) 
   {
       *out_z_flo = -( ( 0x4000 - *out_z ) / 1024.0 );
       asm nop
   }
   else 
   {
       *out_z_flo = *out_z / 1024.0;
       asm nop
   }
}

// ------------------------------------------------------------------------- END

